@@ -7,11 +7,11 @@
 ## http://search.cpan.org/dist/Devel-NYTProf/
 ##
 ###########################################################
-## $Id: NYTProf.pm 1402 2010-11-19 16:45:40Z tim.bunce@gmail.com $
+## $Id$
 ###########################################################
 package Devel::NYTProf;
 
-our $VERSION = '4.06';
+our $VERSION = '4.23'; # also change in Devel::NYTProf::Core
 
 package    # hide the package from the PAUSE indexer
     DB;
@@ -50,7 +50,7 @@ __END__
 
 =head1 NAME
 
-Devel::NYTProf - Powerful fast feature-rich perl source code profiler
+Devel::NYTProf - Powerful fast feature-rich Perl source code profiler
 
 =head1 SYNOPSIS
 
@@ -248,7 +248,8 @@ ranges of all the subroutines.
 NYTProf can profile applications that fork, and does so with no loss of
 performance.
 NYTProf detects the fork and starts writing a new profile file with the pid
-appended to the filename.
+appended to the filename. Since L<nytprofhtml> only works with a single profile
+file you may want to merge multiple files using L<nytprofmerge>.
 
 =head2 Fast Profiling
 
@@ -355,6 +356,10 @@ may be rewritten as
 so the profile won't show a statement count for line 2 in your source code
 because the C<return> was merged into the C<if> statement on the preceding line.
 
+Also 'empty' statements like C<1;> are removed entirely.  Such statements are
+empty because the optimizer has already removed the pointless constant in void
+context. It then goes on to remove the now empty statement (in perl >= 5.13.7).
+
 Using the C<optimize=0> option disables the optimizer so you'll get lower
 overall performance but more accurately assigned statement counts.
 
@@ -455,23 +460,11 @@ likely to change in future.
 
 =head2 usecputime=1
 
-Measure user CPU + system CPU time instead of the real elapsed 'wall clock'
-time (which is the default). But there are better ways to do this, read on.
+This option has been removed. Profiling won't be enabled if set.
 
-Measuring CPU time has the advantage of making the measurements independent of
-time spent blocked waiting for the cpu or network i/o etc. But the method used
-by Cusecputime=1> also has the severe disadvantage of having typically I<far>
-less accurate timings.
-
-Most systems have a 0.01 second granularity in the results from the C<times()>
-sytem call.  With modern processors having multi- gigahertz clocks, 0.01
-seconds is like a lifetime. The 'ticks' of this CPU time clock
-happen so rarely relative to the activity of a most applications that you'd
-have to run the code for many hours to have any hope of reasonably useful results.
-
-A much better alternative is to use the L</clock=N> option to select a
-high-resolution CPU time clock, if available on your system, because that'll
-give you higher resolution and work for the subroutine profiler as well.
+Use the L</clock=N> option to select a high-resolution CPU time clock, if
+available on your system, instead. That will give you higher resolution and work
+for the subroutine profiler as well.
 
 =head2 file=...
 
@@ -1068,10 +1061,6 @@ I<cpu-time> clock instead of a real-time one. That should be much less
 noisy, though you will lose visibility of wait-times due to network
 and disk I/O, for example.
 
-If your system doesn't support the C<clock=N> option then you could try
-using the C<usecputime=1> option. That will give you cpu-time measurements
-but only at a very low 1/100th of a second resolution.
-
 =head1 BUGS
 
 Possibly. All complex software has bugs. Let me know if you find one.
@@ -1104,7 +1093,7 @@ file as a stream of chunks of data.
 
 =head1 TROUBLESHOOTING
 
-=head2 "Profile data incomplete, ..."
+=head2 "Profile data incomplete, ..." or "File format error: ..."
 
 This error message means the file doesn't contain all the expected data.
 That may be because it was truncated (perhaps the filesystem was full) or,
@@ -1122,13 +1111,59 @@ If the sudden death was due to a signal then L</sigexit=1> may help.
 If the sudden death was due to calling C<POSIX::_exit($status)> then you'll
 need to call L</finish_profile> before calling C<POSIX::_exit>.
 
-=head2 Some files don't have profile information
+You'll also get this error if the code trying to read the profile is itself
+being profiled. That's most likely to happen if you enable profiling via the
+C<PERL5OPT> environment variable and have forgotten to unset it.
 
-This is usually due to NYTProf being loaded after the other files, for example
+=head2 Some source files don't have profile information
+
+This is usually due to NYTProf being initialized after some perl files have
+already been compiled.
 
 If you can't alter the command line to add "C<-d:NYTProf>" you could try using
 the C<PERL5OPT> environment variable. See L</PROFILING>.
 
+You could also try using the L</use_db_sub=1> option.
+
+=head2 Eval ... has unknown invoking fid
+
+When using the statement profiler you may see a warning message like this:
+
+  Eval '(eval 2)' (fid 9, flags:viastmt,savesrc) has unknown invoking fid 10
+ 
+Notice that the eval file id (fid 9 in this case) is lower than the file id
+that invoked the eval (fid 10 in this case). This is a known problem caused by
+the way perl works and how the profiler assigns and outputs the file ids.
+The invoking fid is known but gets assigned a fid and output after the fid for
+the eval, and that causes the warning when the file is read.
+
+=head2 Warning: %d subroutine calls had negative time
+
+There are two likely causes for this: clock instability, or accumulated timing
+errors.
+
+Clock instability, if present on your system, is most likely to be noticeable on
+very small/fast subroutines that are called very few times.
+
+Accumulated timing errors can arise because the subroutine profiler uses
+floating point values (NVs) to store the times.  They are most likely to be
+noticed on subroutines that are called a few times but which make a large
+number of calls to very fast subroutines (such as opcodes). In this case the
+accumulated time apparently spent making those calls can be greater than the
+time spent in the calling subroutine.
+
+If you rerun nytprofhtml (etc.) with the L</trace=N> option set >0 you'll see
+trace messages like  "%s call has negative time: incl %fs, excl %fs" for each
+affected subroutine.
+
+Try profiling with the L</slowops=N> option set to 0 to disable the profiling
+of opcodes. Since opcodes often execute in a few microseconds they are a common
+cause of this warning.
+
+You could also try recompiling perl to use 'long doubles' for the NV floating
+point type (use Configure -Duselongdouble). If you try this please let me know.
+I'd also happily take a patch to use long doubles, if available, by default.
+
 =head1 AUTHORS AND CONTRIBUTORS
 
 B<Tim Bunce> (L<http://www.tim.bunce.name> and L<http://blog.timbunce.org>)
@@ -1191,7 +1226,7 @@ Statement-level profilers:
 Devel::NYTProf is a (now distant) fork of Devel::FastProf, which was itself an
 evolution of Devel::SmallProf.
 
-Adam Kaplan took Devel::FastProf and added html report generation (based on
+Adam Kaplan forked Devel::FastProf and added html report generation (based on
 Devel::Cover) and a test suite - a tricky thing to do for a profiler.
 Meanwhile Tim Bunce had been extending Devel::FastProf to add novel
 per-sub and per-block timing, plus subroutine caller tracking.
@@ -1200,7 +1235,7 @@ When Devel::NYTProf was released Tim switched to working on Devel::NYTProf
 because the html report would be a good way to show the extra profile data, and
 the test suite made development much easier and safer.
 
-Then he went a little crazy and added a slew of new features, in addition to
+Then Tim went a little crazy and added a slew of new features, in addition to
 per-sub and per-block timing and subroutine caller tracking. These included the
 'opcode interception' method of profiling, ultra-fast and robust inclusive
 subroutine timing, doubling performance, plus major changes to html reporting
@@ -1212,7 +1247,14 @@ keep NYTProf working with the latest development perl versions. Nicholas Clark
 added zip compression, many optimizations, and C<nytprofmerge>.
 Jan Dubois contributed Windows support.
 
-Adam's work is sponsored by The New York Times Co. L<http://open.nytimes.com>.
-Tim's work was partly sponsored by Shopzilla L<http://www.shopzilla.com> during 2008.
+Adam's work was sponsored by The New York Times Co. L<http://open.nytimes.com>.
+Tim's work was partly sponsored by Shopzilla L<http://www.shopzilla.com> during 2008
+but hasn't been sponsored since then.
+
+For the record, Tim has never worked for the New York Times nor has he received any
+kind of sponsorship or support from them in relation to NYTProf. The name of
+this module is simply result of the history outlined above, which can be
+summarised as: Adam forked an existing module when he added his enhancements
+and Tim didn't.
 
 =cut
